from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui

class RemoveFromDictionaryProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['dictionary', 'keys'])] = self
        #Remove given keys from the dictionary. if key not in it, ignore it.
    def i18n_Proxy(self, func):
        def proxy(self, dictionary, *keys):
            #創出該呼叫的參數紀錄
            full_args = [str(dictionary), str(keys)]

            #翻譯
            #因為dictionary無法直接翻譯，所以拆成keys和values去分別翻譯，回傳值是list
            dict_keys_trans = i18n.I18nListener.MAP.values(list(dictionary.keys()), full_args)
            dict_have_multi_trans  = False
            for dt in dict_keys_trans:
                if len(dt) >1:
                    dict_have_multi_trans  = True

            keys_trans = i18n.I18nListener.MAP.values(keys, full_args)
            keys_have_multi_trans = False
            for lt in keys_trans:
                if len(lt) >1:
                    keys_have_multi_trans = True
                    break
            # logger.warn(keys_have_multi_trans)
            # logger.warn(dict_have_multi_trans)
            
            if keys_have_multi_trans or dict_have_multi_trans:
                RemoveFromDictionaryProxy.show_warning(self, dictionary, keys, full_args) #show warning
                #此case也沒有fail的問題，因為遇到key不在dictionary的話會自動忽略

                # 對預計開啟的UI做一些準備
                i18n.I18nListener.Is_Multi_Trans = True
                
                for i, dt in enumerate(dict_keys_trans):
                    if len(dt)>1 and str(full_args)+list(dictionary.keys())[i] not in ui.UI.unique_log: #FIXME dict keys是否要在這邊判斷
                        multi_trans_word = [list(dictionary.keys())[i]]                                # 還是要移交add_trans_info處理
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multi_trans_word, dt, full_args, func.__name__)
                for i, lt in enumerate(keys_trans):
                        if len(lt) > 1 and str(full_args)+keys[i] not in ui.UI.unique_log:
                            multi_trans_word = [keys[i]]     
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__) #將翻譯詞加進等等UI會用到的dictionary中
            #將dictionary 翻譯過後的 key 合併 
            # 這邊會出錯，因為key要是唯一值， 暫時用原先的key代替
            # dictionary = dict(zip( list(dictionary.keys()), list(dictionary.values()) ) )         
            #將處理好的翻譯回傳給robot原生keyword
            #FIXME 這邊比較麻煩，之後user選擇key的唯一翻譯後，目前還是只會回傳原本的key值
            # logger.warn(dictionary)
            return func(self, dictionary, *keys)
        return proxy

    def show_warning(self, dictionary, keys, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_dict_keys = Proxy().deal_warning_message_for_list(dictionary.keys(), full_args, 'DICT_KEYS')
        message_for_keys = Proxy().deal_warning_message_for_list(keys, full_args, 'KEYS')
        if message_for_keys != '':
            message = language + test_name + message_for_dict_keys + '\n' + \
            message_for_keys + '\n' + 'You should verify translation is correct!'
            logger.warn(message)


import SeleniumLibrary
import inspect
import os
import json
import sys
import itertools
import time
import re
from glob import glob
from selenium import webdriver
from I18nMap import I18nMap
from MappingRoutesGenerator import MappingRoutesGenerator
from SeleniumLibrary.base import keyword
from robot.libraries.BuiltIn import BuiltIn
from robot.libraries.Collections import Collections
from robot.api import logger
from robot.libraries.Screenshot import Screenshot
from selenium.webdriver.remote.webelement import WebElement
from I18nTrigger import I18nTrigger # 此行會執行Ii18nTrigger
# from ManyTranslations import run

class I18nListener:
 
    ROBOT_LISTENER_API_VERSION = 2
    MAP = None
    TRANSLATION_FILE = json.loads("{}") # json -> python
    LOCALE = None
    Not_SHOW_WARNING_WORDS = []
    Is_Multi_Trans=False
    SETTING_KEYS = {}
    SETTING_TRANS = {}
    SETTING_ARGS = {}

    def __init__(self, locale='en-US', language_file_path='.',not_show_warning_words='None'):
        self.is_admin_language_set=False
        self.is_ui_open=False
        self.locale = locale
        self.attrs = {}
        self.locale_dict = {'en-US':'United Kingdom - English', 
                            'ja':'日本 - 日本語', 
                            'ko':'대한민국 - 한국어', 
                            'zh-CN':'中国 - 简体中文', 
                            'zh-TW':'台灣 - 繁體中文',  
                            'de-CH':'Schweiz - Deutsch'}

        #decide language file's path. It's '.' by default. 
        if language_file_path == 'i18njson':
            language_file_path = os.path.normpath(os.path.dirname(os.path.abspath(__file__)))

        MappingRoutesGenerator().generate(language_file_path)
        for f in glob('%s/languageFiles/%s/*%s.json' % (language_file_path, locale, locale)): #各種語言翻譯檔(common-xx.json more-xx.json etc.)
            with open(f, 'r', encoding='UTF-8') as i18n_file: #將json檔打開
                i18n_dict = json.load(i18n_file)  #將裏頭的json結構轉換成python dict
            self.combine_i18n_dict(source_dict=i18n_dict, target_dict=I18nListener.TRANSLATION_FILE)
        I18nListener.MAP = I18nMap(I18nListener.TRANSLATION_FILE, locale) #i18nMAP，傳入擁有翻譯檔dict的TRANSLATION_FILE 和 語言
        I18nListener.LOCALE = locale # for get language Ex zh-TW, zh-CN
        I18nListener.Not_SHOW_WARNING_WORDS = self.parse_not_show_warning_words(not_show_warning_words)

    '''
        append all key, value of source_dict to target_dict
        source_dict is the dict of json file like 'common-zh-TW.json'...
    '''
    def combine_i18n_dict(self, source_dict, target_dict): #寫一份到TRANSLATION_FILE
        for key, value in source_dict.items():
            target_dict[key] = value
    
    def start_suite(self, name, attrs):
        if not self.is_admin_language_set:#set the admin language in the first suite start
            self.is_admin_language_set=True
            BuiltIn().set_global_variable('${language}',self.locale_dict[self.locale])
        with open("./setting.txt", 'a+') as file:
            if os.stat("./setting.txt").st_size != 0:
                file.seek(0)  #這行很重要，將指針指到文件頭
                for i, line in enumerate(file.readlines()):
                    # logger.warn("in for")
                    split_key_value = []
                    split_key_value=line.strip("\n").split('~')
                    read_args = split_key_value[1].split('#')
                    # logger.warn(split_key_value)]
                    #FIXME 這邊將'翻譯詞'(key)當成去取'翻譯' 和 '參數部分'(full_args)的媒介
                    #  但是，此種作法在 有多個相同翻譯詞(例如:多個support)在不同情況下有不同的翻譯，
                    #  便會導致後面的 新翻譯 和 新full_args 將前面的洗掉(因為Dictionary的特性)
                    #  因為 翻譯詞的不唯一性，我不該把其他資料與它連結，必須想另一種可能不借助dict的實作
                    I18nListener.SETTING_KEYS[i] = split_key_value[2]
                    I18nListener.SETTING_TRANS[i] = split_key_value[3]
                    I18nListener.SETTING_ARGS[i] = read_args
                    # logger.warn(read_args)
                    # logger.warn(type(read_args))
                    # logger.warn(type(split_key_value[0]))
                    # logger.warn(I18nListener.SETTING_TRANS)

    def parse_not_show_warning_words(self, words_string):
        if words_string == "Not_show_warning.txt":
            Not_show_warning_txt = glob('%s/Not_show_warning.txt' % (os.path.dirname(os.path.abspath(__file__))))[0]
            with open(Not_show_warning_txt, 'r', encoding='utf-8') as f:
                words_string = f.read()
        words = words_string.split('+')
        return words

    def end_suite(self, name, attrs):
        # 判斷若當前無UI開啟，且滿足有一詞多譯的情況，才需要開啟UI
        # logger.warn(I18nListener.Is_Multi_Trans)
        if not self.is_ui_open and I18nListener.Is_Multi_Trans:
            # logger.warn("index out of range here")
            self.is_ui_open=True
            import ManyTranslations
            ManyTranslations.UI()


import json
import os

class MappingRoutesGenerator:

    def generate(self, language_file_path):
            language_files = os.scandir(os.path.join(language_file_path, 'languageFiles/en-US')) # 'en-US' is a standard language file for this generator 
            parse_route_record = []
            i18n_json_routes_map = {} # Use this dict dump to json file

            def append_route_to_map(last_key_of_route, value): #ex: {'s1':{'s2':5 } } =>last_key_of_route=s2, value=5
                path = ''.join(parse_route_record) + "[\'%s\']" % last_key_of_route
                if value not in i18n_json_routes_map:
                    i18n_json_routes_map[value] = [] 
                i18n_json_routes_map[value].append(path)
                
            def parse_i18n_key_mapping(key_mappings):
                for key in key_mappings.keys():
                    if isinstance(key_mappings[key], dict):  #判斷是否是一個dict
                        parse_route_record.append("['%s']" % key)
                        parse_i18n_key_mapping(key_mappings[key]) #遞迴往下找，因為 key_mappings[key]是一個dictionary
                    else: #value 若不是一個dictionary
                        append_route_to_map(key, key_mappings[key])
                #pop out parent node when all children node had already been append to parse_route_record
                else:
                    if len(parse_route_record) > 0:
                        parse_route_record.pop()
            
            for language_file in language_files:
                with open(os.path.normpath(language_file.path), 'r', encoding='utf-8') as f:
                    language_key_text_mappings = json.load(f) # json-> python
                    parse_i18n_key_mapping(language_key_text_mappings)
            json.dump(i18n_json_routes_map, open('./mappingRoutes.json', "w"),indent=4) 

if __name__=='__main__':
    MappingRoutesGenerator().generate()


from robot.libraries.BuiltIn import BuiltIn
from robot.libraries.Collections import Collections
from selenium import webdriver
import SeleniumLibrary
import inspect
import json

def parse_library(library):
    function_names = {}
    for str_method in dir(library):
        func = getattr(library, str_method)
        if repr(type(func)) == "<class 'function'>":
            function_names[func.__name__] = get_argument_declaration(func)
    generate_json_file('%s.json' % library.__name__, {library.__name__:function_names})

def parse_seleniumLibrary():
    function_names = {}
    for keyword in SeleniumLibrary.SeleniumLibrary().keywords: # Use ".keywords" Can get all keywords
        function_names[keyword] = get_argument_declaration(keyword)
    generate_json_file("SeleniumLibrary.json", {"SeleniumLibrary: ":function_names})

def parse_seleniumLibrary():
    function_names = {}
    keywords = [keyword.replace(' ', '_').lower() for keyword in SeleniumLibrary.SeleniumLibrary().keywords]
    for str_keywords_class in dir(SeleniumLibrary):
        keywords_class = getattr(SeleniumLibrary, str_keywords_class)
        for str_method in dir(keywords_class):
            if str_method.replace(' ', '_').lower() in keywords:
                func = getattr(keywords_class, str_method)
                if repr(type(func)) == "<class 'function'>":
                    function_names[func.__name__] = get_argument_declaration(func)
    generate_json_file("SeleniumLibrary.json", {"SeleniumLibrary: ":function_names})

def generate_json_file(output_file_name, source):
    with open("./result/"+output_file_name, 'w') as file:
        json.dump(source, file, indent=4) #python->json

def get_argument_declaration(func):
    args = inspect.getargspec(func).args[1:]  # [0] is self
    defaults = inspect.getargspec(func).defaults
    varargs = inspect.getargspec(func).varargs
    keywords = inspect.getargspec(func).keywords
    if defaults:
        defaults = ['=' + repr(default) for default in defaults]
        defaults = [''] * (len(args) - len(defaults)) + defaults
        args = list(arg[0] + arg[1] for arg in zip(args, defaults))
    if varargs:
        args.append(varargs)
    if keywords:
        args.append(keywords)
    return args

if __name__ == "__main__":
    parse_seleniumLibrary() 
    parse_library(Collections)
    parse_library(BuiltIn)
    parse_library(webdriver.Chrome)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui

class DictionaryShouldContainItemProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['dictionary', 'key', 'value', 'msg=None'])] = self
        # Dictionary應該包含Item(key : value)
        # 這邊的 dict key 和 key 都交由內層呼叫 dictionaryshouldcontainkey 再做處理
        # 也就是說: 此proxy只會先做'dict value'和'value'部分的翻譯
        #FIXME 目前USER對於key的翻譯要選擇兩次才會自動選，原因是內層呼叫的full_args不同了
    def i18n_Proxy(self, func):
        def proxy(self, dictionary, key ,value, msg=None):            
            #創出該次呼叫的參數紀錄
            full_args = [str(dictionary), key, value] #將dictionary轉str, 方便之後資料讀寫

            #翻譯
            #因為dictionary無法直接翻譯，所以拆成keys和values去分別翻譯，回傳值是list
            dict_values_trans = i18n.I18nListener.MAP.values(list(dictionary.values()), full_args)            
            dict_have_multi_trans = False 
            for dt in dict_values_trans:
                if len(dt) >1:
                    dict_have_multi_trans  = True
            value_trans = i18n.I18nListener.MAP.value(value, full_args)
            
            #遭遇一詞多譯
            if len(value_trans)>1 or dict_have_multi_trans:
                DictionaryShouldContainItemProxy.show_warning(self, dictionary, value, full_args) #show warning
                #檢查case會pass or fail
                if key in dictionary.keys() and dictionary[key] == value: #pass
                    # 對預計開啟的UI做一些準備
                    i18n.I18nListener.Is_Multi_Trans = True
                    
                    for i, dt in enumerate(dict_values_trans):
                        if len(dt)>1 and str(full_args)+list(dictionary.values())[i] not in ui.UI.unique_log: #FIXME dict keys是否要在這邊判斷
                            multi_trans_word = [list(dictionary.values())[i]]                                # 還是要移交add_trans_info處理
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, dt, full_args, func.__name__)
                    if len(value_trans) > 1 and str(full_args)+value not in ui.UI.unique_log:
                        multiple_translation_word = [value]     
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multiple_translation_word, value_trans, full_args, func.__name__) #將翻譯詞加進等等UI會用到的dictionary中
            #將dictionary 翻譯過後的key,value合併 
            # 這邊會出錯，因為key要是唯一值， 暫時用原先的key代替
            dictionary = dict(zip(list(dictionary.keys()), dict_values_trans))         
            #將處理好的翻譯回傳給robot原生keyword
            #key的部分不用擔心，因為裡面還會再呼叫一次DictionaryShouldContainKeyProxy，交給後面去處理
            return func(self, dictionary, key, value_trans,msg)
        return proxy

    def show_warning(self, dictionary, value, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_dict_value = Proxy().deal_warning_message_for_list(dictionary.values(),full_args, 'DICT_VALUE')
        message_for_value = Proxy().deal_warning_message_for_one_word(value, full_args,  'VALUE')
        if message_for_dict_value or message_for_value:
            message = language + test_name + message_for_dict_value + '\n' + \
                        message_for_value + '\n' +\
                        'You should verify translation is correct!'
            logger.warn(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui
from SeleniumLibrary.keywords.selectelement import SelectElementKeywords

class SelectFromListByLabelProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['locator', 'labels'])] = self
    # select options from selection list by labels
    # UnselectFromListByLabel 也適用此方法， 但unselect僅能用在multi-selections
    def i18n_Proxy(self, func):
        def proxy(self, locator, *labels):
            if not labels:                          #檢查機制
                return func(self, locator, labels)
            #創出該呼叫的參數紀錄
            full_args = [locator, str(labels)]

            BuiltIn().import_library('SeleniumLibrary')
            #翻譯， 會翻譯xpath內有需要被翻譯的屬性(邏輯定義在i18nMap)，翻譯完需要屬性後會回傳整條xpath，
            #並會設定multiple_translation_words，讓下一行get_multiple_translation_words()取用
            locator_trans = i18n.I18nListener.MAP.locator(BuiltIn().replace_variables(locator), full_args)
            # logger.warn(locator_trans)
            multiple_translation_words = i18n.I18nListener.MAP.get_multiple_translation_words()
            words_trans = i18n.I18nListener.MAP.values(multiple_translation_words, full_args)

            labels_trans = i18n.I18nListener.MAP.values(labels, full_args)
            labels_have_multi_trans = False
            for lt in labels_trans:
                if len(lt) >1:
                    labels_have_multi_trans = True
                    break

            xpath = ""
            #遭遇一詞多譯
            if len(locator_trans)>1 or labels_have_multi_trans:
                SelectFromListByLabelProxy.show_warning(self, multiple_translation_words, labels, full_args) #show warning
                #對翻譯過後可能的多種xpath做串接
                for i, lt in enumerate(locator_trans):
                    xpath += '|' + lt.replace('xpath', '') if i!=0 else lt.replace('xpath', '')

                #判斷case會過還是fail (使用原生library)
                all_options = SelectElementKeywords._get_options(self, locator)
                all_labels = SelectElementKeywords._get_labels(self, all_options)
                is_pass = False
                for lt in labels_trans:
                    for single_tran in lt:
                        if single_tran in all_labels:
                            is_pass = True
                            break

                if is_pass: # pass
                    # 對預計開啟的UI做一些準備
                    i18n.I18nListener.Is_Multi_Trans = True
                    
                    for i, word_trans in enumerate(words_trans):
                        if len(word_trans)>1 and str(full_args)+multiple_translation_words[i] not in ui.UI.unique_log:
                            multi_trans_word = [multiple_translation_words[i]] 
                            ui.UI.origin_xpaths_or_arguments.append(full_args)                               
                            ui.UI.add_trans_info(self, multi_trans_word, word_trans, full_args, func.__name__)
                    for i, lt in enumerate(labels_trans):
                        if len(lt) > 1 and str(full_args)+labels[i] not in ui.UI.unique_log:
                            multi_trans_word = [labels[i]]     
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__) #將翻譯詞加進等等UI會用到的dictionary中
            else: #沒有一詞多譯
                xpath = locator_trans[0]
            #將處理好的翻譯回傳給robot原生keyword
            #這邊labels是tuple可以用'*' unpack argument，但labels_trans內部item還是list
            #為了下面回傳時好處理，此處必須把"list包list"的一詞多譯壓縮成一個string
            
            # 若有翻譯會使case過，則用其置換labels_trans中的翻譯 #FIXME 可優化
            all_options = SelectElementKeywords._get_options(self, locator)
            all_labels = SelectElementKeywords._get_labels(self, all_options)
            for i, lt in enumerate(labels_trans): 
                    for single_tran in lt:
                        if single_tran in all_labels:
                            labels_trans[i] = single_tran
                            break

            # logger.warn(expected_trans)
            return func(self, BuiltIn().replace_variables(xpath), *tuple(labels_trans))
        return proxy

    def show_warning(self, multi_trans_words, labels, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_words = Proxy().deal_warning_message_for_list(multi_trans_words,full_args, 'MULTI_TRANS_WORDS')
        message_for_labels = Proxy().deal_warning_message_for_list(labels, full_args, 'LABELS')
        if message_for_words or message_for_labels:
            message = language + test_name + message_for_words + '\n' + \
                message_for_labels + '\n' +'You should verify translation is correct!'
            logger.warn(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui
from SeleniumLibrary.keywords.tableelement import TableElementKeywords

class TableColumnShouldContainProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['locator', 'column', 'expected', 'loglevel=\'TRACE\''])] = self
        #<table>
    def i18n_Proxy(self, func):
        def proxy(self, locator, column, expected, loglevel='TRACE'):
            #創出該呼叫的參數紀錄
            full_args = [locator, column, expected]

            #翻譯
            expected_trans = i18n.I18nListener.MAP.value(expected, full_args)

            locator_trans = i18n.I18nListener.MAP.locator(BuiltIn().replace_variables(locator), full_args)
            multiple_translation_words = i18n.I18nListener.MAP.get_multiple_translation_words()
            word_trans = i18n.I18nListener.MAP.values(multiple_translation_words, full_args)

            xpath = ''
            my_expected = ''
            # logger.warn(locator_trans)
            #遭遇一詞多譯            
            if len(expected_trans)>1 or len(locator_trans)>1:
                TableColumnShouldContainProxy.show_warning(self, multiple_translation_words, expected, full_args) #show warning

                #檢查case會pass or fail
                for i, lt in enumerate(locator_trans):
                    is_pass = False
                    is_actual = BuiltIn().run_keyword_and_return_status('Get WebElement', lt) #如果畫面上有該翻譯的element存在
                    if is_actual:
                        xpath += lt.replace('xpath:','')
                        for et in expected_trans:
                            if TableElementKeywords._find_by_column(self, lt, column, et) is not None:
                                # 因為expected_trans在一詞多譯的情況下可能不只一種，
                                # 直接回傳會導致case出錯，所以目前打算在proxy先測試並取唯一值再回傳
                                my_expected += et
                                is_pass = True
                                break

                    if is_pass: #pass
                        # 對預計開啟的UI做一些準備
                        i18n.I18nListener.Is_Multi_Trans = True
                        
                        for i, wt in enumerate(word_trans):
                            if len(wt)>1 and str(full_args)+multiple_translation_words[i] not in ui.UI.unique_log: #FIXME dict keys是否要在這邊判斷
                                multi_trans_word = [multiple_translation_words[i]]                            # 還是要移交add_trans_info處理
                                ui.UI.origin_xpaths_or_arguments.append(full_args)
                                ui.UI.add_trans_info(self, multi_trans_word, wt, full_args, func.__name__)
                        if len(expected_trans) > 1 and str(full_args)+expected not in ui.UI.unique_log:
                            multiple_translation_word = [expected]     
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multiple_translation_word, expected_trans, full_args, func.__name__) #將翻譯詞加進等等UI會用到的dictionary中
                        break
            else: #沒有一詞多譯的情況
                xpath = locator_trans[0].replace('xpath:','')
                my_expected = expected_trans[0]

            #將處理好的翻譯回傳給robot原生keyword
            return func(self, xpath, column, my_expected, loglevel)
        return proxy

    def show_warning(self, multiple_translation_words, expected, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_multi_trans_words = Proxy().deal_warning_message_for_list(multiple_translation_words, full_args, 'MULTI_TRANS_WORDS')
        message_for_expected = Proxy().deal_warning_message_for_one_word(expected, full_args, 'EXPECTED')
        if message_for_multi_trans_words or message_for_expected:
            message = language + test_name + message_for_multi_trans_words + '\n' +\
                      message_for_expected + '\n' + 'You should verify translation is correct!'
            logger.warn(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui

class DictionaryShouldContainKeyProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['dictionary', 'key', 'msg=None'])] = self
        # DictionaryShouldContainItem內部也會呼叫到此proxy
    def i18n_Proxy(self, func):
        def proxy(self, dictionary, key, msg=None):
            #創出該次呼叫的參數紀錄
            full_args = [str(dictionary), key] #將dictionary轉str, 方便之後資料讀寫

            #翻譯
            #因為dictionary無法直接翻譯，所以拆成keys和values去分別翻譯，回傳值是list
            dict_keys_trans = i18n.I18nListener.MAP.values(list(dictionary.keys()), full_args)
            dict_have_multi_trans  = False
            for dt in dict_keys_trans:
                if len(dt) >1:
                    dict_have_multi_trans  = True
                    break
            key_trans = i18n.I18nListener.MAP.value(key, full_args)

            #遭遇一詞多譯
            if len(key_trans)>1 or dict_have_multi_trans:
                DictionaryShouldContainKeyProxy.show_warning(self, dictionary, key, full_args) #show warning
                #檢查case會pass or fail
                is_pass = False
                if 'not' in func.__name__:
                    if key not in dictionary.keys():
                        is_pass = True
                else:
                    if key in dictionary.keys():
                        is_pass = True
                if is_pass: #pass
                    # 對預計開啟的UI做一些準備
                    i18n.I18nListener.Is_Multi_Trans = True
                    
                    for i, dt in enumerate(dict_keys_trans):
                        if len(dt)>1 and str(full_args)+list(dictionary.keys())[i] not in ui.UI.unique_log: #FIXME dict keys是否要在這邊判斷
                            multi_trans_word = [list(dictionary.keys())[i]]                                # 還是要移交add_trans_info處理
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multi_trans_word, dt, full_args, func.__name__)
                    if len(key_trans) > 1 and str(full_args)+key not in ui.UI.unique_log:
                        multiple_translation_word = [key]     
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multiple_translation_word, key_trans, full_args, func.__name__) #將翻譯詞加進等等UI會用到的dictionary中
            #將dictionary 翻譯過後的key合併 
            # 這邊會出錯，因為key要是唯一值， 暫時用原先的key代替
            # dictionary = dict(zip(list(dictionary.keys()), dictionary.values()))         
            #將處理好的翻譯回傳給robot原生keyword
            #FIXME 這邊比較麻煩，之後user選擇key的唯一翻譯後，目前還是只會回傳原本的key值
            # logger.warn(dictionary)
            return func(self, dictionary, key, msg)
        return proxy

    def show_warning(self, dictionary,key, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_dict_key = Proxy().deal_warning_message_for_list(dictionary.keys(),full_args, 'DICT_KEY')
        message_for_key = Proxy().deal_warning_message_for_one_word(key, full_args,  'KEY')
        if message_for_dict_key or message_for_key:
            message = language + test_name + message_for_dict_key + '\n' + message_for_key + '\n'\
                        'You should verify translation is correct!'
            logger.warn(message)


from robot.libraries.BuiltIn import BuiltIn
from robot.api.deco import keyword
from selenium import webdriver

class self_util():
    @keyword(name='Scroll To The End')
    def scroll_to_the_end(self):
        selenium = BuiltIn().get_library_instance('SeleniumLibrary')
        selenium.driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")




import SeleniumLibrary
import re
import os
import inspect
import json
import sys
from glob import glob
import SeleniumLibrary
from selenium import webdriver
from robot.libraries.Collections import Collections
from SeleniumLibrary.base import keyword
from robot.libraries.BuiltIn import BuiltIn

class I18nTrigger:

    def __init__(self):
        self.arg_format = {}
        self.new_proxy_instance()
        self.set_proxy_func_to_library_class_func(Collections)
        self.set_proxy_func_to_library_class_func(webdriver.Chrome)
        self.set_proxy_func_to_library_class_func(BuiltIn)
        self.set_proxy_func_to_SeleniumLibrary()

    def get_module_name(self, path):
        file = re.findall('[_A-Za-z]+.py', path)   # file是含有['']的檔名 , file[0]為檔名
        file_name = re.findall('[_A-Za-z]+', file[0]) #filename是將檔名和py分開的串列
        module_name = file_name[0]      #filename[0]是不含.py的檔名
        return module_name

    def get_class_name(self, text):
        m = re.search('class[ A-Za-z]+', text)
        class_define =  m.group(0)   #class_define是class+class名稱
        m = re.findall('[a-zA-Z]+', class_define)
        class_name = m[1] # Fisrst is 'Class' Second is Class name
        return class_name

    def new_proxy_instance(self):
        module_names = []
        class_names = []
        for f in glob('%s\proxyContainer\*.py' % (os.path.dirname(os.path.abspath(__file__)))):
            with open(f, 'r', encoding='UTF-8') as sub_proxy:
                text = sub_proxy.read()     #text是.py檔的檔名+內容
                module_name =  self.get_module_name(sub_proxy.name) #sub_proxy.name是包含路徑的檔名
                if text and module_name != '__init__'and module_name != 'Proxy': # 不處理父類別
                    module_names.append(module_name)   #將檔名append在一起
                    class_names.append(self.get_class_name(text))   #將class名append在一起

        zipped = zip ( module_names , class_names )  #將兩者打包起來
        for arg in zipped:
            inport_text = 'from proxyContainer import %s' %(arg[0]) # from proxyContainer import 模組名
            exec(inport_text) #動態執行
            instance_text = '%s.%s(self.arg_format)' %(arg[0], arg[1]) # x.x(self.arg_format)
            eval(instance_text)  #計算instance_text

    #get proxy
    def get_func_proxy(self, func): #重要!用來判斷該keyword是否有代理keyword存在
        args_declaration = self.get_argument_declaration(func) #得出該func的參數宣告部分
        if repr(args_declaration) in list(self.arg_format.keys()):  #***如果參數宣告在arg_format這個dict的keys中
            return self.arg_format[repr(args_declaration)].i18n_Proxy(func)  #回傳arg_format中value(是proxy的類別名).i18n_Proxy(func)
        return func #否則回傳原本的func屬性值

    def get_argument_declaration(self, func):
        args = inspect.getfullargspec(func).args[1:]  # [0] is self, get出該func的參數部分去掉self
        defaults = inspect.getfullargspec(func).defaults #get出該func參數的default部分
        varargs = inspect.getfullargspec(func).varargs
        keywords = inspect.getargspec(func).keywords
        if defaults:            # 幫args加上default值，如果有的話
            defaults = ['=' + repr(default) for default in defaults]
            defaults = [''] * (len(args) - len(defaults)) + defaults
            args = list(arg[0] + arg[1] for arg in zip(args, defaults))
        if varargs:
            args.append(varargs)
        if keywords:
            args.append(keywords)
        return args

    def set_proxy_func_to_library_class_func(self, library):
        for str_method in dir(library):  # dir(library)是library的全部屬性
            func = getattr(library, str_method) #func是該屬性的屬性值
            if repr(type(func)) == "<class 'function'>":  #如果屬性值是 function
                setattr(library, str_method, self.get_func_proxy(func))

    # provide a surrogate of selenium keywords and functions so that they can test i18n
    ## SeleniumLibrary
    def set_proxy_func_to_SeleniumLibrary(self):
        import SeleniumLibrary
        keywords = [keyword.replace(' ', '_').lower() for keyword in SeleniumLibrary.SeleniumLibrary().keywords] #把seleniumlibrary內的keyword資訊格式化成只顯示其keyword名稱
        for str_keywords_class in dir(SeleniumLibrary): #Seleniumlibrary內的各種class
            keywords_class = getattr(locals().get(SeleniumLibrary.__name__), str_keywords_class) #得出class在seleniumlibrary內的呼叫路徑
            for str_method in dir(keywords_class): #dir(keywords_class)是該class內的各種method
                if str_method.replace(' ', '_').lower() in keywords: #若method名稱出現在keywords list內(和keyword同名)
                    func = getattr(keywords_class, str_method)
                    if repr(type(func)) == "<class 'function'>":#如果屬性值是 function
                        setattr(keywords_class, str_method, keyword(self.get_func_proxy(func)))

I18nTrigger() #import I18nTrigger that it will run this line.


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui
from robot.utils import is_string, is_list_like

class ShouldContainProxy(Proxy):   #container要包含item才算pass
    def __init__(self, arg_format):
        arg_format[repr(['container', 'item', 'msg=None', 'values=True', 'ignore_case=False'])] = self
                        # container:可能是string或list,  item:通常是string
                        # ShouldNotContain也會呼叫此keyword
    def i18n_Proxy(self, func):
        def proxy(self, container, item, msg=None, values=True, ignore_case=False):
            #創出該呼叫的參數紀錄
            full_args = [str(container), item] #container有機會是list，轉str，方便之後資料讀寫
            # logger.warn(container)

            #翻譯
            container_trans = i18n.I18nListener.MAP.values(container, full_args)
            item_trans = i18n.I18nListener.MAP.value(item, full_args)
            # logger.warn(item_trans)
            # logger.warn(container_trans)

            container_have_multi_trans = False
            if is_list_like(container):
                for lt in container_trans:
                    if len(lt) >1:
                        container_have_multi_trans  = True
                        break 
            elif is_string(container):
                if len(container_trans)>1:
                    container_have_multi_trans = True

            #遭遇一詞多譯
            if container_have_multi_trans or len(item_trans)>1:
                ShouldContainProxy.show_warning(self, container, item, full_args) #show warning
                #檢查case會pass or fail
                #FIXME 因為container有可能是抓畫面上翻譯過網頁的值，所以這邊的判斷
                #      要考慮container和item不是同語言的情形
                is_pass = False
                if 'not' in func.__name__ :
                    if is_string(container):
                        container = container.lower()
                        if item not in container and (index not in container for index in item_trans):
                            is_pass=True
                    elif is_list_like(container):
                        # 若不同語言的情況下，item不在container內，有可能同語言下卻會包含
                        # 所以要考慮item和item_trans，皆不在container中
                        if item not in container and (index not in container for index in item_trans):
                            is_pass = True
                else:
                    if is_string(container):
                        container = container.lower()
                        if item in container or (index in container for index in item_trans):
                            is_pass = True
                    elif is_list_like(container):
                        if item in container or (index in container for index in item_trans):
                            is_pass = True

                if is_pass: #pass
                    # 對預計開啟的UI做一些準備
                    i18n.I18nListener.Is_Multi_Trans = True

                    if is_list_like(container):
                        for i, lt in enumerate(container_trans):
                            # logger.warn(container)
                            # logger.warn(container[i])
                            if len(lt)>1 and str(full_args)+container[i] not in ui.UI.unique_log: #FIXME dict keys是否要在這邊判斷
                                multi_trans_word = [container[i]]                            # 還是要移交add_trans_info處理
                                ui.UI.origin_xpaths_or_arguments.append(full_args)
                                ui.UI.add_trans_info(self, multi_trans_word, lt, full_args, func.__name__)
                    elif is_string(container):
                        if len(container_trans)>1 and str(full_args)+container not in ui.UI.unique_log:
                            multiple_translation_word = [container]     
                            ui.UI.origin_xpaths_or_arguments.append(full_args)
                            ui.UI.add_trans_info(self, multiple_translation_word, container_trans, full_args, func.__name__) #將翻譯詞加進等等UI會用到的dictionary中
                    if len(item_trans)>1 and str(full_args)+item not in ui.UI.unique_log:
                        multiple_translation_word = [item]     
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multiple_translation_word, item_trans, full_args, func.__name__) #將翻譯詞加進等等UI會用到的dictionary中

            #將處理好的翻譯回傳給robot原生keyword
            #  原本預期會pass,因為有可能container是抓畫面上翻譯過網頁的值，
            #  所以item_trans在一詞多譯的情況下，可能不會包含於container內，
            #  而導致case出錯，目前是打算在proxy先測試過再做回傳
            if 'not' not in func.__name__ :
                if is_list_like(item_trans):
                    for it in item_trans: #把item_trans一詞多譯換成會過的那個
                        if [it] in container_trans:
                            item_trans = [it]
                            break
            #FIXME should not contain還未定義，未來有機率出錯
            return func(self, container_trans, item_trans, msg)
        return proxy                        
    
    def show_warning(self, container, item, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_container = Proxy().deal_warning_message_for_list(container, full_args, 'CONTAINER')
        message_for_item = Proxy().deal_warning_message_for_one_word(item, full_args, 'Expected Contain')
        if message_for_container or message_for_item :
            message = language + test_name + message_for_container + '\n' +  message_for_item + '\n' + 'You should verify translation is correct!'
            logger.warn(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui

class TitleShouldBeProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['title', 'message=None'])] = self
        # 會和網頁視窗的title比較是否相同
    
    def i18n_Proxy(self, func):
        def proxy(self, title, message=None):
            #創出該呼叫的參數紀錄
            full_args = [title]

            #翻譯，如果有一詞多譯的話要藉由判斷，找出正確的翻詞回傳
            title_trans = i18n.I18nListener.MAP.value(title, full_args)
            # logger.warn(title_trans)

            #遭遇一詞多譯
            if len(title_trans)>1:
                TitleShouldBeProxy.show_warning(self, title, full_args) #show warning
                #檢查case會pass or fail
                is_pass = False
                actual = self.get_title()
                for tt in title_trans:
                    if tt == actual:
                        is_pass = True
                        break
                if is_pass: #pass
                    # 對預計開啟的UI做一些準備
                    i18n.I18nListener.Is_Multi_Trans = True

                    if len(title_trans) > 1 and str(full_args)+title not in ui.UI.unique_log:
                        multiple_translation_word = [title]     
                        ui.UI.origin_xpaths_or_arguments.append(full_args)
                        ui.UI.add_trans_info(self, multiple_translation_word, title_trans, full_args, func.__name__) #將翻譯詞加進等等UI會用到的dictionary中
            #將處理好的翻譯回傳給robot原生keyword
            #因為畫面上的title是唯一值，如果直接將title_trans回傳，有可能發生錯誤
            #所以要先做過濾的動作
            actual = self.get_title()
            for tt in title_trans:
                if tt == actual:
                    title_trans = tt
            return func(self, title_trans, message)
        return proxy

    def show_warning(self, title, full_args):
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_title = Proxy().deal_warning_message_for_one_word(title, full_args, 'TITLE')
        if message_for_title :
            message = language + test_name + message_for_title + '\n' + 'You should verify translation is correct!'
            logger.warn(message)


from .Proxy import Proxy
from robot.libraries.BuiltIn import BuiltIn
import sys
from robot.libraries.Screenshot import Screenshot
from robot.api import logger
import I18nListener as i18n
import ManyTranslations as ui

class ShouldBeEqualProxy(Proxy):
    def __init__(self, arg_format):
        arg_format[repr(['first', 'second', 'msg=None', 'values=True', 'ignore_case=False', 'formatter=\'str\'' ])] = self
    
    def i18n_Proxy(self, func):
        def proxy(self, first, second, msg=None, values=True, ignore_case=False, formatter='str'):
            # logger.warn("ShouldBeEqual")
            if 'not' in func.__name__:
                compare = lambda x,y:True if x != y else False
            else:
                compare = lambda x,y:True if x == y else False

            #創出該呼叫的參數紀錄
            full_args = [first, second]

            #翻譯
            first_trans = i18n.I18nListener.MAP.value(first, full_args)
            second_trans = i18n.I18nListener.MAP.value(second, full_args)

            #遭遇一詞多譯
            if len(first_trans) >1 or len(second_trans) > 1 : 
                # logger.warn("have multi trans")
                ShouldBeEqualProxy.show_warning(self, first, second, full_args)  

                ##判斷case會過還是fail 
                
                for ft in first_trans:
                    for st in second_trans:
                        if compare(ft,st): #
                            # FIXME 現在會fail的話不顯示UI(但會過的話會把"所有"翻譯詞顯示在UI上)(參考下面實作)
                            # 並且也會把完整xpath或arguments顯示在UI上
                            i18n.I18nListener.Is_Multi_Trans = True

                            if len(first_trans) > 1 and str(full_args)+first not in ui.UI.unique_log:
                                multiple_translation_word = []     
                                multiple_translation_word.append(first) #將expected word包裝成list格式
                                ui.UI.origin_xpaths_or_arguments.append(full_args)
                                ui.UI.add_trans_info(self, multiple_translation_word, first_trans, full_args, func.__name__) #將翻譯詞加進等等UI會用到的dictionary中

                            if len(second_trans) > 1 and str(full_args)+second not in ui.UI.unique_log:
                                multiple_translation_word = []     
                                multiple_translation_word.append(second) #將expected word包裝成list格式
                                ui.UI.origin_xpaths_or_arguments.append(full_args)
                                ui.UI.add_trans_info(self, multiple_translation_word, second_trans, full_args, func.__name__) #將翻譯詞加進等等UI會用到的dictionary中
                            return func(self, ft, st, msg, values, ignore_case, formatter)
            return func(self, first_trans[0], second_trans[0], msg, values, ignore_case, formatter) 
            #執行到這行可能有兩種情況: 1.沒有一詞多譯(回傳第一組翻譯，也就是原本的翻譯) 2.一詞多譯的翻譯詞找不到會讓case過的(回傳第一組翻譯)
        return proxy
    
    def deal_translate_message(self, value, full_args, message_title):
        translation = i18n.I18nListener.MAP.value(value, full_args)[0]
        if value != translation:
            message = ('%s argument evaluates to' + ' ' + '\'%s\'' + ' is translated to: ' + '\'%s\'') %(message_title, value, translation)  + '\n'
        else:
            message = ('%s argument evaluates to' + ' ' + '\'%s\'') %(message_title, value)  + '\n'
        return message

    def show_warning(self, first, second, full_args): 
        language = 'i18n in %s:\n ' %i18n.I18nListener.LOCALE
        test_name = ('Test Name: %s') %BuiltIn().get_variable_value("${TEST NAME}") + '=> Exist multiple translations of the word' + '\n'
        message_for_first = Proxy().deal_warning_message_for_one_word(first, full_args,  'First')
        message_for_second = Proxy().deal_warning_message_for_one_word(second, full_args, 'Second')
        message = language + test_name + message_for_first + ' '*3 + '\n' +  message_for_second + '\n' + 'You should verify translation is correct!'
        if message_for_first or message_for_second:
            logger.warn(message)
        else:
            message_for_first = ShouldBeEqualProxy.deal_translate_message(self, first, full_args,'First')
            message_for_second = ShouldBeEqualProxy.deal_translate_message(self, second, full_args,'Second')
            message = language +' ' + message_for_first + ' '*2 +  message_for_second
            logger.info(message)
